Package: triplediff
Title: Triple-Difference Estimators
Version: 0.2.4
Authors@R: c(person("Marcelo", "Ortiz-Villavicencio", email = "marcelo.ortiz@emory.edu", role = c("aut", "cre")),
              person("Pedro H. C.", "Sant'Anna", email = "pedro.santanna@emory.edu", role = c("aut"))
            )
Description: Implements triple-difference (DDD) estimators for both average
              treatment effects and event-study parameters. Methods include regression
              adjustment, inverse-probability weighting, and doubly-robust estimators,
              all of which rely on a conditional DDD parallel-trends assumption and
              allow covariate adjustment across multiple pre- and post-treatment
              periods. The methodology is detailed in Ortiz-Villavicencio and
              Sant'Anna (2025) <doi:10.48550/arXiv.2505.09942>.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.3)
RoxygenNote: 7.3.3
Imports: BMisc (>= 1.4.9), data.table (>= 1.15.0), fastglm (>= 0.0.3),
        Matrix (>= 1.6.1), parallel (>= 1.4.0), Rcpp (>= 1.0.12)
LinkingTo: Rcpp (>= 1.0.12)
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
URL: http://marcelortiz.com/triplediff/
NeedsCompilation: yes
Packaged: 2026-06-13 20:03:54 UTC; marcelortiz
Author: Marcelo Ortiz-Villavicencio [aut, cre],
  Pedro H. C. Sant'Anna [aut]
Maintainer: Marcelo Ortiz-Villavicencio <marcelo.ortiz@emory.edu>
Repository: CRAN
Date/Publication: 2026-06-13 20:20:02 UTC
