# Generated by roxygen2: do not edit by hand

S3method(print,agg_ddd)
S3method(print,ddd)
S3method(summary,agg_ddd)
S3method(summary,ddd)
export(agg_ddd)
export(att_dr)
export(att_dr_rc)
export(compute_aggregation)
export(ddd)
export(gen_dgp_2periods)
export(gen_dgp_mult_periods)
export(process_attgt)
import(BMisc)
import(data.table)
import(fastglm)
import(stats)
importFrom(Matrix,Matrix)
importFrom(Rcpp,sourceCpp)
importFrom(stats,model.matrix)
importFrom(stats,update)
importFrom(utils,write.table)
useDynLib(triplediff)
