# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mboot_did <- function(linrep, nboot) {
    .Call('_triplediff_mboot_did', PACKAGE = 'triplediff', linrep, nboot)
}

checkPartitionUniqueness <- function(dta, idname, partitionName) {
    invisible(.Call('_triplediff_checkPartitionUniqueness', PACKAGE = 'triplediff', dta, idname, partitionName))
}

checkTreatmentUniqueness <- function(dta, idname, treatName) {
    invisible(.Call('_triplediff_checkTreatmentUniqueness', PACKAGE = 'triplediff', dta, idname, treatName))
}

checkWeightsUniqueness <- function(dta, idname) {
    invisible(.Call('_triplediff_checkWeightsUniqueness', PACKAGE = 'triplediff', dta, idname))
}

